#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Copyright (c) 2014, Perceptive Automation, LLC. All rights reserved.
# http://www.indigodomo.com

import re

def validateKey(key):
    p = re.compile('\A(?!XML)[a-z][a-z0-9]*$', re.IGNORECASE)
    return bool(p.match(key))
    
################################################################################
class Plugin(indigo.PluginBase):
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		super(Plugin, self).__init__(pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		self.debug = self.pluginPrefs.get("showDebugInfo", False)
		if self.debug:
			indigo.server.log("Debugging enabled")

	########################################
	def startup(self):
		self.debugLog(u"startup called")

	def shutdown(self):
		self.debugLog(u"shutdown called")

	########################################
	def propList(self, filter="", valuesDict=None, typeId="", targetId=0):
		self.debugLog(u"propList called with valuesDict:\n%s" % unicode(valuesDict))
		try:
			device = indigo.devices.get(int(valuesDict["deviceId"]), None)
			self.debugLog(u"propList: deviceProps: \n%s" % unicode(device.pluginProps))
			propList = [(key, "%s : %s" % (key,value)) for key, value in device.pluginProps.items()]
		except:
			propList = []
		return propList

	########################################
	def selectDevice(self, valuesDict, typeId="", devId=None):
		self.debugLog("selectDevice called")
		valuesDict["isPropSelected"] = False
		valuesDict["editKey"] = ""
		valuesDict["previousKey"] = ""
		valuesDict["editValue"] = ""
		valuesDict["propertyKey"] = ""
		return valuesDict

	########################################
	def selectProp(self, valuesDict, typeId="", devId=None):
		self.debugLog("selectProp called with valuesDict:\n%s" % unicode(valuesDict))
		try:
			device = indigo.devices.get(int(valuesDict["deviceId"]), None)
			valuesDict["editKey"] = valuesDict["propertyKey"]
			valuesDict["previousKey"] = valuesDict["propertyKey"]
			valuesDict["editValue"] = device.pluginProps[valuesDict["propertyKey"]]
			valuesDict["isPropSelected"] = True
		except:
			pass
		return valuesDict

	########################################
	def toggleDebugging(self):
		if self.debug:
			indigo.server.log("Turning off debug logging")
			self.pluginPrefs["showDebugInfo"] = False
		else:
			indigo.server.log("Turning on debug logging")
			self.pluginPrefs["showDebugInfo"] = True
		self.debug = not self.debug

	########################################
	def addProperty(self, valuesDict, typeId="", devId=None):
		errorsDict = indigo.Dict()
		try:
			device = indigo.devices.get(int(valuesDict["deviceId"]), None)
			deviceProps = device.pluginProps
			key = valuesDict['addKey']
			if validateKey(key):
				if key not in device.globalProps["com.indigodomo.indigoserver"]:
					deviceProps[key] = valuesDict['addValue']
					device.replacePluginPropsOnServer(deviceProps)
					valuesDict["addKey"] = ""
					valuesDict["addValue"] = ""
				else:
					errorsDict["addKey"] = "Key already exists in the generic properties"					
			else:
				errorsDict["addKey"] = "Key is not valid - it must be alphanumeric and begin with an alphabetic character"
		except:
			pass
		return valuesDict, errorsDict

	########################################
	def updateProperty(self, valuesDict, typeId="", devId=None):
		errorsDict = indigo.Dict()
		try:
			device = indigo.devices.get(int(valuesDict["deviceId"]), None)
			deviceProps = device.pluginProps
			key = valuesDict['editKey']
			previousKey = valuesDict['previousKey']
			value = valuesDict['editValue']
			if key != previousKey:
				if key in deviceProps:
					errorsDict['editKey'] = "New key (%s) already exists in the global properties, please use a different key value" % key
					valuesDict['editKey'] = previousKey
				else:
					del deviceProps[previousKey]
			else:
				deviceProps[key] = value
			deviceProps[key] = value
			if not len(errorsDict):
				device.replacePluginPropsOnServer(deviceProps)
				valuesDict["previousKey"] = key
# 				valuesDict["propertyKey"] = ""
		except:
			pass
		return valuesDict, errorsDict

	########################################
	def deleteProperty(self, valuesDict, typeId="", devId=None):
		try:
			device = indigo.devices.get(int(valuesDict["deviceId"]), None)
			deviceProps = device.pluginProps
			del deviceProps[valuesDict['propertyKey']]
			device.replacePluginPropsOnServer(deviceProps)
		except:
			pass
		valuesDict["editKey"] = ""
		valuesDict["previousKey"] = ""
		valuesDict["editValue"] = ""
		valuesDict["propertyKey"] = ""
		return valuesDict
